package de.hsel.itech.db;

import de.hsel.itech.db.pojo.Address;
import de.hsel.itech.db.pojo.User;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * handles all database stuff for addresses of users.
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.5
 **/
public class AddressDB {

    private Database database;

    AddressDB(Database database) {
        this.database = database;
    }

    /**
     * get all addresses of specified user.
     *
     * @param user user to get addresses from
     * @return list of users addresses
     */
    @NotNull
    public List<Address> getAll(@NotNull User user) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetsByValue(database.tableUserAddress, "user", String.valueOf(user.getId()));
        assert entry != null;

        List<Address> list = new ArrayList<>();

        try {
            ResultSet rs = entry.getKey();
            while(rs.next()) {
                list.add(new Address(user, rs.getLong("id"), rs.getString("name"), rs.getString("street"), rs.getString("number"), rs.getInt("zipCode"), rs.getString("city")));
            }

            entry.getValue().close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return list;
    }

    /**
     * get specific address.
     *
     * @param id address id
     * @return specific address
     */
    @Nullable
    public Address get(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tableAddress, id);
        assert entry != null;

        try {
            ResultSet rs = entry.getKey();
            Address address = new Address(getUserFromId(id), id, rs.getString("name"), rs.getString("street"), rs.getString("number"), rs.getInt("zipCode"), rs.getString("city"));
            entry.getValue().close();

            return address;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * get user from address id.
     *
     * @param id address id
     * @return User
     */
    @Nullable
    private User getUserFromId(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetByValue(database.tableUserAddress, "address", id);
        assert entry != null;

        try {
            User user =  database.user().get(entry.getKey().getLong("user"));
            entry.getValue().close();
            return user;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * insert address into database.
     *
     * @param address address to insert
     * @return insert count
     */
    public int insert(@NotNull Address address) {

        Connection connection = database.getConnection();
        int insertCount = 0;

        try {
            assert connection != null;
            PreparedStatement statement = connection.prepareStatement("INSERT INTO address (name, street, number, zipCode, city) VALUES (?, ?, ?, ?, ?)");
            statement.setString(1, address.getName());
            statement.setString(2, address.getStreet());
            statement.setString(3, address.getNumber());
            statement.setInt(4, address.getZipCode());
            statement.setString(5, address.getCity());

            insertCount += statement.executeUpdate();

            PreparedStatement relationStatement = connection.prepareStatement("INSERT INTO user_address (user, address) VALUES (?, ?)");
            relationStatement.setLong(1, address.getUser().getId());
            relationStatement.setLong(2, address.getId());

            insertCount += statement.executeUpdate();

            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return insertCount;
    }

    /**
     * delete address from database.
     *
     * @param address address to delete
     * @return deletion count
     */
    public int delete(@NotNull Address address) {
        return database.delete(address.getId(), database.tableUserAddress);
    }
}